package net.mikekohn.janitor2;

import androidx.appcompat.app.AppCompatActivity;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.graphics.Color;
import android.os.Bundle;
import android.os.ParcelUuid;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.Set;
import java.util.UUID;

public class MainActivity extends AppCompatActivity implements SensorEventListener
{
  private TextView data;
  private SensorManager sensor_manager;
  private Sensor accelerometer;
  private Sensor magnetometer;
  private Sensor game_rotation;
  private final float[] gravity = new float[3];
  private final float[] geomagnetic = new float[3];
  private final float[] rotation_r = new float[9];
  private final float[] rotation_adjusted = new float[9];
  private final float[] rotation_i = new float[9];
  private final float[] orientation = new float[3];
  private volatile int position = 0;
  private BluetoothSocket socket = null;
  private OutputStream out = null;
  private InputStream in = null;
  private Button button;

  @Override
  protected void onCreate(Bundle savedInstanceState)
  {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    button = findViewById(R.id.button);
    data = findViewById(R.id.data);

    sensor_manager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);

    button.setOnClickListener(new View.OnClickListener()
    {
      @Override
      public void onClick(View view)
      {
        Thread thread = new Thread()
        {
          public void run()
          {
            runSequence();
          }
        };

        thread.start();
      }
    });
  }

  @Override
  protected void onResume()
  {
    super.onResume();

    setupBluetooth();

/*
    accelerometer = sensor_manager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);

    if (accelerometer != null)
    {
      boolean result = sensor_manager.registerListener(
        this,
        accelerometer,
        SensorManager.SENSOR_DELAY_NORMAL);
        //SensorManager.SENSOR_DELAY_NORMAL,
        //SensorManager.SENSOR_DELAY_UI);

      System.out.println("accelerometer listener: " + result);
    }

    magnetometer = sensor_manager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);

    if (magnetometer != null)
    {
      boolean result = sensor_manager.registerListener(
        this,
        magnetometer,
        SensorManager.SENSOR_DELAY_NORMAL,
        SensorManager.SENSOR_DELAY_UI);
    }
*/

    game_rotation = sensor_manager.getDefaultSensor(Sensor.TYPE_GAME_ROTATION_VECTOR);

    if (game_rotation != null)
    {
      boolean result = sensor_manager.registerListener(
        this,
        game_rotation,
        SensorManager.SENSOR_DELAY_GAME);
    }
      else
    {
      System.out.println("Error: Missing sensor.");
    }
  }

  @Override
  protected void onPause()
  {
    super.onPause();

    sensor_manager.unregisterListener(this);
  }

  @Override
  public void onAccuracyChanged(Sensor sensor, int accuracy)
  {
  }

  @Override
  public void onSensorChanged(SensorEvent event)
  {
    if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER)
    {
      System.arraycopy(
        event.values,
        0,
        gravity,
        0,
        gravity.length);
    }
      else
    if (event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD)
    {
      System.arraycopy(
        event.values,
        0,
        geomagnetic,
        0,
        geomagnetic.length);

      updateOrientation();
    }
      else
    if (event.sensor.getType() == Sensor.TYPE_GAME_ROTATION_VECTOR)
    {
      position = (int)Math.toDegrees(event.values[0]);

      data.setText("pos: " + position);

    }
  }

  private void updateOrientation()
  {
    SensorManager.getRotationMatrix(
      rotation_r,
      null,
      gravity,
      geomagnetic);

    SensorManager.remapCoordinateSystem(
      rotation_r,
      SensorManager.AXIS_Y,
      SensorManager.AXIS_MINUS_X,
      rotation_adjusted);

    SensorManager.getOrientation(rotation_adjusted, orientation);

    position = (int)Math.toDegrees(orientation[0]);

    data.setText("pos: " + position);

    if (position >= -5 && position <= 5)
    {
      button.setBackgroundColor(Color.RED);
    }
      else
    if (position >= 170 || position <= -170)
    {
      button.setBackgroundColor(Color.GREEN);
    }
      else
    {
      button.setBackgroundColor(Color.BLUE);
    }
  }

  private void setupBluetooth()
  {
    BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
    BluetoothDevice device = null;

    if (adapter != null)
    {
      Set<BluetoothDevice> devices = adapter.getBondedDevices();
      BluetoothDevice[] bluetooth_devices = new BluetoothDevice[devices.size()];

      int i = 0;
      Iterator iter = devices.iterator();

      while (iter.hasNext())
      {
        bluetooth_devices[i] = (BluetoothDevice)iter.next();

        String name = bluetooth_devices[i].getName();
        String address = bluetooth_devices[i].getAddress();

        if (name.equals("linvor"))
        {
          device = bluetooth_devices[i];
          name = name += "*";
        }

        System.out.println("\"" + name + "\" -- " + address);

        i++;
      }

      if (device == null) { return; }

      System.out.println("Connecting to " + device.getName());

      try
      {
        ParcelUuid[] uuids = device.getUuids();

        //socket = bluetooth_devices[position].createRfcommSocketToServiceRecord(
        //  UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"));
        socket = device.createRfcommSocketToServiceRecord(uuids[0].getUuid());
        socket.connect();
        out = socket.getOutputStream();
        in = socket.getInputStream();
      }
      catch (Exception e)
      {
        System.out.println(e.toString());
      }
    }
  }

  public void pause(int milliseconds)
  {
    try
    {
      Thread.sleep(milliseconds);
    }
    catch (Exception e)
    {
      System.out.println(e.toString());
    }
  }

  public void sendCommand(char value)
  {
    try
    {
      out.write((byte)value);
    }
    catch (Exception e)
    {
      System.out.println(e.toString());
    }
  }

  public void rotateLeftTo(int value)
  {
    sendCommand('l');

    while (true)
    {
      int p = position;

      if (p > 90) { p = -180 - (180 - p); }
      if (p <= value) { break; }
    }

    sendCommand('s');
  }

  public void rotateRightTo(int value)
  {
    sendCommand('r');

    while (true)
    {
      int p = position;

      if (p > 90) { p = -180 - (180 - p); }
      if (p >= value) { break; }
    }

    sendCommand('s');
  }

  public void rotateLeftByTime(int milliseconds)
  {
    sendCommand('l');
    pause(milliseconds);
    sendCommand('s');
  }

  public void rotateRightByTime(int milliseconds)
  {
    sendCommand('r');
    pause(milliseconds);
    sendCommand('s');
  }

  public void dropCaster()
  {
    sendCommand('d');
    sendCommand('f');
    pause(200);
    sendCommand('s');
    pause(1000);
  }

  public void runSequence()
  {
    pause(4000);

    for (int count = 0; count < 2; count++)
    {
      // Move forward for 3 seconds.
      sendCommand('f');
      pause(3000);
      sendCommand('s');
      pause(500);

      // Rotate around to the left.
      dropCaster();
      //rotateLeftTo(-90);
      rotateLeftByTime(1500);
      sendCommand('f');
      pause(300);
      sendCommand('s');
      //rotateLeftTo(-180);
      rotateLeftByTime(1500);
      pause(1000);

      // Raise caster.
      sendCommand('u');
      pause(1000);

      // Move forward for 3 seconds.
      sendCommand('f');
      pause(3000);
      sendCommand('s');
      pause(500);

      // Rotate around to the right.
      dropCaster();
      //rotateRightTo(-90);
      rotateRightByTime(1500);
      sendCommand('f');
      pause(300);
      sendCommand('s');
      //rotateRightTo(0);
      rotateRightByTime(1500);
      pause(1000);

      // Raise caster.
      sendCommand('u');
      pause(1000);
    }
  }
}

